@@ -305,7 +305,7 @@ mod_from_mpq (mpz_t r, mpq_t q, mpz_t N, int verbose)
           {
             if (verbose > 1)
               printf ("Warning: factor found during initialization\n");
-            printf ("********** Factor found during step 1: ");
+            printf ("********** Factor found in step 1: ");
           }
 	mpz_out_str (stdout, 10, r);
 	if (verbose > 0)
@@ -388,7 +388,7 @@ main (int argc, char *argv[])
   double maxmem = 0.;
   double stage1time = 0.;
   ecm_params params;
-  int param = ECM_PARAM_DEFAULT; /* -1 means default parametrization */
+  int param = ECM_PARAM_DEFAULT; /* automatic choice */
   char *savefile_s = NULL;
   char *loadfile_s = NULL;
 #ifdef HAVE_GWNUM
@@ -1035,6 +1035,12 @@ main (int argc, char *argv[])
   /* Open resume file for reading, if resuming is requested */
   if (resumefilename != NULL)
     {
+      /* -resume should not be used with -gpu */
+      if (use_gpu)
+        {
+          fprintf (stderr, "Error, -resume not allowed with -gpu\n");
+          exit (EXIT_FAILURE);
+        }
       if (strcmp (resumefilename, "-") == 0)
         resumefile = stdin;
       else
@@ -1323,6 +1329,7 @@ main (int argc, char *argv[])
           fprintf (stderr, "Error, invalid -param value: %d\n", param);
           exit (EXIT_FAILURE);
         }
+      /* params->param might be set if we resume from a file */
       if (params->param == ECM_PARAM_DEFAULT)
           params->param = param;
 
@@ -1342,8 +1349,8 @@ main (int argc, char *argv[])
               exit (EXIT_FAILURE);
             }
           else if (verbose >= OUTPUT_VERBOSE)
-              fprintf (stdout, "Reading batch product (of %zu bits) of "
-                               "primes below B1=%1.0f from %s took %ldms\n", 
+              fprintf (stdout, "Reading batch product (of %"PRIu64" bits) of "
+                               "primes up to B1=%1.0f from %s took %ldms\n", 
                                mpz_sizeinbase (params->batch_s, 2), B1,
                                loadfile_s, cputime () - st);
         }
@@ -1444,7 +1451,7 @@ main (int argc, char *argv[])
          operation */
       mpmod_selftest (n.n);
 #endif
-      
+
       /* now call the ecm library */
       if (result == ECM_NO_FACTOR_FOUND)
 	  /* if torsion was used, some factor may have been found... */
@@ -1492,7 +1499,7 @@ main (int argc, char *argv[])
         }
 
       /* if quiet mode, prints remaining cofactor after last curve */
-      if ((cnt == 0) && (verbose == 0))
+      if ((cnt == 0) && (verbose == 0) && (mpz_cmp_ui (n.n, 1) > 0))
         {
           if (n.cpExpr)
             printf ("%s", n.cpExpr);
