@@ -1037,6 +1037,10 @@ mpmod_init_set (mpmod_t r, const mpmod_t modulus)
     {
       mpz_init2 (r->aux_modulus, Nbits);
       mpz_set (r->aux_modulus, modulus->aux_modulus);
+      /* for ECM_MOD_REDC, ensure r->aux_modulus has n limbs */
+      _mpz_realloc (r->aux_modulus, n);
+      MPN_ZERO (PTR(r->aux_modulus) + ABSIZ(r->aux_modulus),
+                n - ABSIZ(r->aux_modulus));
     }
   if (modulus->repr == ECM_MOD_MODMULN)
     {
@@ -1078,6 +1082,7 @@ mpres_pow_sqr (mpres_t a, const mpres_t b, mpmod_t modulus)
     ecm_sqrredc_basecase (a, b, modulus);
   else
     {
+      ASSERT(modulus->repr == ECM_MOD_REDC);
       mpz_mul (modulus->temp1, b, b);
       REDC (a, modulus->temp1, modulus->temp2, modulus);
     }
@@ -1099,13 +1104,14 @@ mpres_pow_mul (mpres_t a, const mpres_t b, const mpres_t c, mpmod_t modulus)
     ecm_mulredc_basecase (a, b, c, modulus);
   else
     {
+      ASSERT(modulus->repr == ECM_MOD_REDC);
       mpz_mul (modulus->temp1, b, c);
       REDC (a, modulus->temp1, modulus->temp2, modulus);
     }
 }
 
 /* R <- BASE^EXP mod modulus.
-   EXP can be negative.
+   Warning: EXP can be negative (only occurs for P-1 in revision 2982).
  */
 void 
 mpres_pow (mpres_t R, const mpres_t BASE, const mpz_t EXP, mpmod_t modulus)
@@ -1123,17 +1129,23 @@ mpres_pow (mpres_t R, const mpres_t BASE, const mpz_t EXP, mpmod_t modulus)
     {
       size_t expidx;
       mp_limb_t bitmask, expbits;
+      int sgn = mpz_sgn (EXP);
+      mpz_t exp;
 
       /* case EXP=0 */
-      if (mpz_sgn (EXP) == 0)
+      if (sgn == 0)
         {
           mpres_set_ui (R, 1UL, modulus); /* set result to 1 */
           ASSERT_NORMALIZED (R);
           return;
         }
 
-      expidx = mpz_size (EXP) - 1;         /* point at most significant limb */
-      expbits = mpz_getlimbn (EXP, expidx); /* get most significant limb */
+      /* exp <- |EXP| */
+      PTR(exp) = PTR(EXP);
+      SIZ(exp) = ABSIZ(EXP);
+
+      expidx = mpz_size (exp) - 1;         /* point at most significant limb */
+      expbits = mpz_getlimbn (exp, expidx); /* get most significant limb */
       ASSERT (expbits != 0);
 
       /* Scan for the MSB in expbits */
@@ -1144,7 +1156,7 @@ mpres_pow (mpres_t R, const mpres_t BASE, const mpz_t EXP, mpmod_t modulus)
       /* bitmask is set to mask in the msb of expbits */
 
       k = 1; /* sliding window size */
-      expnbits = mpz_sizeinbase (EXP, 2);
+      expnbits = mpz_sizeinbase (exp, 2);
       /* the average number of multiplications is 2^(k-1) + expnbits / (k+1) */
       while ((1 << (k-1)) + expnbits / (k + 1) > (1 << k) + expnbits / (k + 2))
         k ++;
@@ -1191,9 +1203,9 @@ mpres_pow (mpres_t R, const mpres_t BASE, const mpz_t EXP, mpmod_t modulus)
                     }
                   /* if the current bit is the smallest one of the window,
                      we multiply by BASE^w */
-                  if ((mpz_sgn (EXP) > 0 && mpz_scan1 (EXP, n0) == expnbits - 1)
-                    || (mpz_sgn (EXP) < 0 && mpz_scan0 (EXP, n0) == expnbits - 1))
+                  if (mpz_scan1 (exp, n0) == expnbits - 1)
                     {
+                      ASSERT(w/2 < K);
                       mpres_pow_mul (modulus->temp2, (w == 1) ? BASE : B[w/2],
                                      modulus->temp2, modulus);
                       w = lw = 0;
@@ -1203,9 +1215,10 @@ mpres_pow (mpres_t R, const mpres_t BASE, const mpz_t EXP, mpmod_t modulus)
           if (expidx == 0)		/* if we just processed the least */
             break;			/* significant limb, we are done */
           expidx --;
-          expbits = mpz_getlimbn (EXP, expidx);
+          expbits = mpz_getlimbn (exp, expidx);
           bitmask = (mp_limb_t) 1 << (GMP_NUMB_BITS - 1);
         }
+      ASSERT(w/2 < K);
       if (w != 0)
         mpres_pow_mul (modulus->temp2, (w == 1) ? BASE : B[w/2],
                        modulus->temp2, modulus);
@@ -1216,10 +1229,8 @@ mpres_pow (mpres_t R, const mpres_t BASE, const mpz_t EXP, mpmod_t modulus)
 
       /* mpz_getlimbn() ignores sign of argument, so we computed BASE^|EXP|.
          If EXP was negative, do a modular inverse */
-      if (mpz_sgn (EXP) < 0)
-        {
-          mpres_invert (R, R, modulus);
-        }
+      if (sgn < 0)
+        mpres_invert (R, R, modulus);
     } /* if (modulus->repr == ECM_MOD_BASE2 || ... ) */
   ASSERT_NORMALIZED (R);
 }
@@ -1483,7 +1494,7 @@ mpres_sqr (mpres_t R, const mpres_t S1, mpmod_t modulus)
   ASSERT_NORMALIZED (S1);
 
 #ifdef WANT_ASSERT_EXPENSIVE
-  mpz_t test1, test2, test_result1, test_result2;
+  mpz_t test1, test_result1, test_result2;
   ASSERT_ALWAYS (S1 != modulus->temp1 && R != modulus->temp1);
   mpz_init (test1);
   mpz_init (test_result1);
@@ -1687,11 +1698,11 @@ mpres_set_z_for_gcd (mpres_t R, const mpz_t S, mpmod_t modulus)
   ASSERT_NORMALIZED (R);  
 }
 
-/* Compagnon function to mpres_set_z_for_gcd(). It divides by c^n. 
+/* Companion function to mpres_set_z_for_gcd(). It divides by c^n. 
    In case of MULREDC with k b-bit words, c = 1/2^(b*k), so we multiply 
    by 2^(n*b*k). The purpose is to fix products of n terms collected by 
    using mpres_set_z_for_gcd(), so that we can still get the exact residue. */
-   
+
 void 
 mpres_set_z_for_gcd_fix (mpres_t R, const mpres_t S, const mpz_t n, mpmod_t modulus)
 {
@@ -1706,6 +1717,7 @@ mpres_set_z_for_gcd_fix (mpres_t R, const mpres_t S, const mpz_t n, mpmod_t modu
           mpz_init (nb);
           mpres_init (po2, modulus);
 
+          /* Note: modulus->bits is always non-negative for MODMULN and REDC */
           mpz_mul_ui (nb, n, modulus->bits);
           mpres_set_ui (po2, 2, modulus);
           mpres_pow (po2, po2, nb, modulus);
@@ -2029,6 +2041,18 @@ mpres_gcd (mpz_t R, const mpres_t S, const mpmod_t modulus)
   mpz_gcd (R, S, modulus->orig_modulus);
 }
 
+#ifdef HAVE_ADDLAWS
+/* Returns non-zero if the two residues are equal, 
+   and zero if they are not */
+int 
+mpres_equal (const mpres_t S1, const mpres_t S2, mpmod_t modulus)
+{
+  mpz_mod (modulus->temp1, S1, modulus->orig_modulus);
+  mpz_mod (modulus->temp2, S2, modulus->orig_modulus);
+  return (mpz_cmp (modulus->temp1, modulus->temp2) == 0);
+}
+#endif
+
 #if 0 /* those routines are not called in normal operation */
 
 void 
@@ -2111,7 +2135,7 @@ mpres_muldivbysomething_si (mpres_t R, const mpres_t S, const long n,
 
 /* Returns 1 if successful, 0 if not */
 static int
-test_mpres_set_z_for_gcd_fix(const int maxk, mpmod_t modulus)
+test_mpres_set_z_for_gcd_fix (const int maxk, mpmod_t modulus)
 {
   mpres_t m, prod;
   mpz_t n;
@@ -2152,16 +2176,6 @@ test_mpres_set_z_for_gcd_fix(const int maxk, mpmod_t modulus)
   return 1;
 }
 
-/* Returns non-zero if the two residues are equal, 
-   and zero if they are not */
-int 
-mpres_equal (const mpres_t S1, const mpres_t S2, mpmod_t modulus)
-{
-  mpz_mod (modulus->temp1, S1, modulus->orig_modulus);
-  mpz_mod (modulus->temp2, S2, modulus->orig_modulus);
-  return (mpz_cmp (modulus->temp1, modulus->temp2) == 0);
-}
-
 int
 mpmod_selftest (const mpz_t n)
 {
